@@ -288,6 +288,7 @@ test_tls_import (NMVpnPluginUiInterface *plugin, const char *dir)
 	test_item ("tls-import-data", s_vpn, NM_OPENVPN_KEY_LOCAL_IP, NULL);
 	test_item ("tls-import-data", s_vpn, NM_OPENVPN_KEY_REMOTE_IP, NULL);
 	test_item ("tls-import-data", s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
+	test_item ("tls-import-data", s_vpn, NM_OPENVPN_KEY_TLS_REMOTE, "/CN=myvpn.company.com");
 
 	expected_path = g_strdup_printf ("%s/keys/mg8.ca", dir);
 	test_item ("tls-import-data", s_vpn, NM_OPENVPN_KEY_CA, expected_path);
@@ -349,6 +350,115 @@ test_tls_export (NMVpnPluginUiInterface *plugin, const char *dir)
 	g_free (path);
 }
 
+static void
+test_pkcs12_import (NMVpnPluginUiInterface *plugin, const char *dir)
+{
+	NMConnection *connection;
+	NMSettingConnection *s_con;
+	NMSettingIP4Config *s_ip4;
+	NMSettingVPN *s_vpn;
+	const char *expected_id = "pkcs12";
+	char *expected_path;
+
+	connection = get_basic_connection ("pkcs12-import", plugin, dir, "pkcs12.ovpn");
+	ASSERT (connection != NULL, "pkcs12-import", "failed to import connection");
+
+	/* Connection setting */
+	s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
+	ASSERT (s_con != NULL,
+	        "pkcs12-import", "missing 'connection' setting");
+
+	ASSERT (strcmp (nm_setting_connection_get_id (s_con), expected_id) == 0,
+	        "pkcs12-import", "unexpected connection ID");
+
+	ASSERT (nm_setting_connection_get_uuid (s_con) == NULL,
+	        "pkcs12-import", "unexpected valid UUID");
+
+	/* IP4 setting */
+	s_ip4 = (NMSettingIP4Config *) nm_connection_get_setting (connection, NM_TYPE_SETTING_IP4_CONFIG);
+	ASSERT (s_ip4 == NULL,
+	        "pkcs12-import", "unexpected 'ip4-config' setting");
+
+	/* VPN setting */
+	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
+	ASSERT (s_vpn != NULL,
+	        "pkcs12-import", "missing 'vpn' setting");
+
+	/* Data items */
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE, NM_OPENVPN_CONTYPE_TLS);
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_TAP_DEV, NULL);
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_PROTO_TCP, NULL);
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_COMP_LZO, "yes");
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_RENEG_SECONDS, NULL);
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_REMOTE, "173.8.149.245");
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_PORT, "1194");
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_STATIC_KEY, NULL);
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, NULL);
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_CIPHER, NULL);
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_LOCAL_IP, NULL);
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_REMOTE_IP, NULL);
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_AUTH, NULL);
+
+	expected_path = g_strdup_printf ("%s/keys/mine.p12", dir);
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_CA, expected_path);
+	g_free (expected_path);
+
+	expected_path = g_strdup_printf ("%s/keys/mine.p12", dir);
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_CERT, expected_path);
+	g_free (expected_path);
+
+	expected_path = g_strdup_printf ("%s/keys/mine.p12", dir);
+	test_item ("pkcs12-import-data", s_vpn, NM_OPENVPN_KEY_KEY, expected_path);
+	g_free (expected_path);
+
+	/* Secrets */
+	test_secret ("pkcs12-import-secrets", s_vpn, NM_OPENVPN_KEY_PASSWORD, NULL);
+	test_secret ("pkcs12-import-secrets", s_vpn, NM_OPENVPN_KEY_CERTPASS, NULL);
+
+	g_object_unref (connection);
+}
+
+#define PKCS12_EXPORTED_NAME "pkcs12.ovpntest"
+static void
+test_pkcs12_export (NMVpnPluginUiInterface *plugin, const char *dir)
+{
+	NMConnection *connection;
+	NMConnection *reimported;
+	char *path;
+	gboolean success;
+	GError *error = NULL;
+	int ret;
+
+	connection = get_basic_connection ("pkcs12-export", plugin, dir, "pkcs12.ovpn");
+	ASSERT (connection != NULL, "pkcs12-export", "failed to import connection");
+
+	path = g_build_path ("/", dir, PKCS12_EXPORTED_NAME, NULL);
+	success = nm_vpn_plugin_ui_interface_export (plugin, path, connection, &error);
+	if (!success) {
+		if (!error)
+			FAIL ("pkcs12-export", "export failed with missing error");
+		else
+			FAIL ("pkcs12-export", "export failed: %s", error->message);
+	}
+
+	/* Now re-import it and compare the connections to ensure they are the same */
+	reimported = get_basic_connection ("pkcs12-export", plugin, dir, PKCS12_EXPORTED_NAME);
+	ret = unlink (path);
+	ASSERT (connection != NULL, "pkcs12-export", "failed to re-import connection");
+
+	/* Clear secrets first, since they don't get exported, and thus would
+	 * make the connection comparison below fail.
+	 */
+	remove_secrets (connection);
+
+	ASSERT (nm_connection_compare (connection, reimported, NM_SETTING_COMPARE_FLAG_EXACT) == TRUE,
+	        "pkcs12-export", "original and reimported connection differ");
+
+	g_object_unref (reimported);
+	g_object_unref (connection);
+	g_free (path);
+}
+
 static void
 test_non_utf8_import (NMVpnPluginUiInterface *plugin, const char *dir)
 {
@@ -384,7 +494,7 @@ test_non_utf8_import (NMVpnPluginUiInterface *plugin, const char *dir)
 	        "non-utf8-import", "missing 'vpn' setting");
 
 	expected_path = g_strdup_printf ("%s/%s", dir, expected_cacert);
-	test_item ("tls-import-data", s_vpn, NM_OPENVPN_KEY_CA, expected_path);
+	test_item ("non-utf8-import-data", s_vpn, NM_OPENVPN_KEY_CA, expected_path);
 	g_free (expected_path);
 
 	g_object_unref (connection);
@@ -492,6 +602,148 @@ test_static_key_export (NMVpnPluginUiInterface *plugin, const char *dir)
 	g_free (path);
 }
 
+static void
+test_port_import (NMVpnPluginUiInterface *plugin,
+                  const char *detail,
+                  const char *dir,
+                  const char *file,
+                  const char *expected_id,
+                  const char *expected_port)
+{
+	NMConnection *connection;
+	NMSettingConnection *s_con;
+	NMSettingVPN *s_vpn;
+
+	connection = get_basic_connection (detail, plugin, dir, file);
+	ASSERT (connection != NULL, detail, "failed to import connection");
+
+	/* Connection setting */
+	s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
+	ASSERT (s_con != NULL,
+	        detail, "missing 'connection' setting");
+
+	ASSERT (strcmp (nm_setting_connection_get_id (s_con), expected_id) == 0,
+	        detail, "unexpected connection ID");
+
+	/* VPN setting */
+	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
+	ASSERT (s_vpn != NULL,
+	        detail, "missing 'vpn' setting");
+
+	/* Data items */
+	test_item (detail, s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE, NM_OPENVPN_CONTYPE_TLS);
+	test_item (detail, s_vpn, NM_OPENVPN_KEY_PORT, expected_port);
+
+	g_object_unref (connection);
+}
+
+static void
+test_port_export (NMVpnPluginUiInterface *plugin,
+                  const char *detail,
+                  const char *dir,
+                  const char *file,
+                  const char *exported_name)
+{
+	NMConnection *connection;
+	NMConnection *reimported;
+	char *path;
+	gboolean success;
+	GError *error = NULL;
+	int ret;
+
+	connection = get_basic_connection (detail, plugin, dir, file);
+	ASSERT (connection != NULL, detail, "failed to import connection");
+
+	path = g_build_path ("/", dir, exported_name, NULL);
+	success = nm_vpn_plugin_ui_interface_export (plugin, path, connection, &error);
+	if (!success) {
+		if (!error)
+			FAIL (detail, "export failed with missing error");
+		else
+			FAIL (detail, "export failed: %s", error->message);
+	}
+
+	/* Now re-import it and compare the connections to ensure they are the same */
+	reimported = get_basic_connection (detail, plugin, dir, exported_name);
+	ret = unlink (path);
+	ASSERT (connection != NULL, detail, "failed to re-import connection");
+
+	/* Clear secrets first, since they don't get exported, and thus would
+	 * make the connection comparison below fail.
+	 */
+	remove_secrets (connection);
+
+	ASSERT (nm_connection_compare (connection, reimported, NM_SETTING_COMPARE_FLAG_EXACT) == TRUE,
+	        detail, "original and reimported connection differ");
+
+	g_object_unref (reimported);
+	g_object_unref (connection);
+	g_free (path);
+}
+
+static void
+test_tun_opts_import (NMVpnPluginUiInterface *plugin, const char *dir)
+{
+	NMConnection *connection;
+	NMSettingVPN *s_vpn;
+
+	connection = get_basic_connection ("tunopts-import", plugin, dir, "tun-opts.conf");
+	ASSERT (connection != NULL, "tunopts-import", "failed to import connection");
+
+	/* VPN setting */
+	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
+	ASSERT (s_vpn != NULL,
+	        "tunopts-import", "missing 'vpn' setting");
+
+	/* Data items */
+	test_item ("tunopts-import-data", s_vpn, NM_OPENVPN_KEY_MSSFIX, "yes");
+	test_item ("tunopts-import-data", s_vpn, NM_OPENVPN_KEY_TUNNEL_MTU, "1300");
+	test_item ("tunopts-import-data", s_vpn, NM_OPENVPN_KEY_FRAGMENT_SIZE, "1200");
+
+	g_object_unref (connection);
+}
+
+#define TUNOPTS_EXPORTED_NAME "tun-opts.ovpntest"
+static void
+test_tun_opts_export (NMVpnPluginUiInterface *plugin, const char *dir)
+{
+	NMConnection *connection;
+	NMConnection *reimported;
+	char *path;
+	gboolean success;
+	GError *error = NULL;
+	int ret;
+
+	connection = get_basic_connection ("tunopts-export", plugin, dir, "tun-opts.conf");
+	ASSERT (connection != NULL, "tunopts-export", "failed to import connection");
+
+	path = g_build_path ("/", dir, TUNOPTS_EXPORTED_NAME, NULL);
+	success = nm_vpn_plugin_ui_interface_export (plugin, path, connection, &error);
+	if (!success) {
+		if (!error)
+			FAIL ("tunopts-export", "export failed with missing error");
+		else
+			FAIL ("tunopts-export", "export failed: %s", error->message);
+	}
+
+	/* Now re-import it and compare the connections to ensure they are the same */
+	reimported = get_basic_connection ("tunopts-export", plugin, dir, TUNOPTS_EXPORTED_NAME);
+	ret = unlink (path);
+	ASSERT (connection != NULL, "tunopts-export", "failed to re-import connection");
+
+	/* Clear secrets first, since they don't get exported, and thus would
+	 * make the connection comparison below fail.
+	 */
+	remove_secrets (connection);
+
+	ASSERT (nm_connection_compare (connection, reimported, NM_SETTING_COMPARE_FLAG_EXACT) == TRUE,
+	        "tunopts-export", "original and reimported connection differ");
+
+	g_object_unref (reimported);
+	g_object_unref (connection);
+	g_free (path);
+}
+
 int main (int argc, char **argv)
 {
 	GError *error = NULL;
@@ -521,11 +773,23 @@ int main (int argc, char **argv)
 	test_tls_import (plugin, argv[1]);
 	test_tls_export (plugin, argv[1]);
 
+	test_pkcs12_import (plugin, argv[1]);
+	test_pkcs12_export (plugin, argv[1]);
+
 	test_non_utf8_import (plugin, argv[1]);
 
 	test_static_key_import (plugin, argv[1]);
 	test_static_key_export (plugin, argv[1]);
 
+	test_port_import (plugin, "port-import", argv[1], "port.ovpn", "port", "2345");
+	test_port_export (plugin, "port-export", argv[1], "port.ovpn", "port.ovpntest");
+
+	test_port_import (plugin, "rport-import", argv[1], "rport.ovpn", "rport", "6789");
+	test_port_export (plugin, "rport-export", argv[1], "rport.ovpn", "rport.ovpntest");
+
+	test_tun_opts_import (plugin, argv[1]);
+	test_tun_opts_export (plugin, argv[1]);
+
 	g_object_unref (plugin);
 
 	basename = g_path_get_basename (argv[0]);
